/* base/src/drv drvBb902.c,v 1.3 2018/11/28 00:38:13 anj Exp */
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory.
* burrBrown is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/
/*
 * subroutines that are used to interface to the binary output cards
 *
 * 	Author:      Bob Dalesio
 * 	Date:        5-26-88
 *
 * Modification Log:
 * -----------------
 * .01	10-31-91	bg	broke bb902 code out of bo_driver.c 
 * .02  02-20-92        bg      Added level to io_report.  Added ability
 *                              to read out raw values on card if level
 *                              > 0.
 * .03	08-10-92	joh	made number of cards runtime configurable 
 * .04	08-25-92	mrk	made masks a macro
 * .05	09-14-93	mrk	Let report just display one hex value
 *				 
 */

/*
 * Code Portions:
 *
 * bo_drv_init	Finds and initializes all binary output cards present
 * bo_driver	Interfaces to the binary output cards present
 */

#include <vxWorks.h>
#include <stdlib.h>
#include <stdio.h>
#include <vxLib.h>
#include <sysLib.h>

#include "vme.h"
#include "drvSup.h"
#include <epicsExport.h>

#include "drvBb902.h"

static short bo_num_cards = 4;
static unsigned short bo_addrs = 0xd800;


struct {
        long    number;
        DRVSUPFUN       report;
        DRVSUPFUN       init;
} drvBb902={
        2,
        bb902_io_report,
        bb902_driver_init};
epicsExportAddress(drvet,drvBb902);

#define MAX_BB_BO_CARDS (bo_num_cards)

/* Burr-Brown 902 binary output memory structure */
struct bo_bb902{
        short   csr;                    /* control status register */
        unsigned short   low_value;              /* low order 16 bits value */
        unsigned short   high_value;             /* high order 16 bits value */
        char    end_pad[0x100-6];       /* pad until next card */
};

static char *bb902_shortaddr;

/* pointers to the binary output cards */
struct bo_bb902	**pbo_bb902s;	/* Burr-Brown 902s */


/*
 * BO_DRIVER_INIT
 *
 * intialization for the binary output cards
 */
long bb902_driver_init(){
	int bomode;
        int status;
	short	i;
	struct bo_bb902	*pbo_bb902;

	pbo_bb902s = (struct bo_bb902 **)calloc(MAX_BB_BO_CARDS,
						sizeof(*pbo_bb902s));
	if(!pbo_bb902s){
		return ERROR;
	}

	/* intialize the Burr-Brown 902 binary output cards */

	/* base address of the burr-brown 902 binary output cards */
     
        status = sysBusToLocalAdrs(VME_AM_SUP_SHORT_IO,(char *)(long)bo_addrs,(char **)&bb902_shortaddr);
        if (status != OK){ 
           printf("Addressing error in bb902 driver\n");
           return (ERROR);
        }
        pbo_bb902 = (struct bo_bb902 *)bb902_shortaddr;
	/* determine which cards are present */
	for (i = 0; i < bo_num_cards; i++,pbo_bb902++){
	    if (vxMemProbe((char *)pbo_bb902,READ,sizeof(short),(char *)&bomode) == OK)
		pbo_bb902s[i] = pbo_bb902;
	    else
		pbo_bb902s[i] = 0;
	}
        return (0);

}

/*
 * BB902_DRIVER
 *
 * interface to the Burr-Brown binary outputs
 */

long bb902_driver(short card, u_long val, u_long mask)
{
	unsigned long work;

	/* verify card exists */
	if (!pbo_bb902s[card])
		return (-1);
	/* use structure to handle high and low short swap */
	/* get current output */
	work = (pbo_bb902s[card]->high_value << 16) /* high */
		+ pbo_bb902s[card]->low_value;	    /* low */
	/* alter specified bits */
	work = (work & ~mask) | (val & mask);

	/* write new output */
	pbo_bb902s[card]->high_value = (unsigned short)(work >> 16);
		pbo_bb902s[card]->low_value = (unsigned short)work;
	return (0);
}

/*
 * bb902_read
 * 
 * read the binary output
 */
long bb902_read(short card, u_long mask, u_long *pval)
{

	/* verify card exists */
	if (!pbo_bb902s[card]) return (-1);
	/* readback */
	*pval = (pbo_bb902s[card]->high_value << 16)	/* high */
	+ pbo_bb902s[card]->low_value;		/* low */
	*pval &= mask;
	return(0);
}

long bb902_io_report(int level)
{
   unsigned long value;
   int		 card;

   for (card = 0; card < MAX_BB_BO_CARDS; card++){
	if (pbo_bb902s[card]){
	   value = (pbo_bb902s[card]->high_value << 16) 
		+ pbo_bb902s[card]->low_value;
           printf("BO: BB902:      card %d value=0x%8.8lx\n",card,value);
        }
   }  
   return 0;
}

int bb902Config(unsigned int ncards, unsigned int base)
{
    bo_num_cards = ncards;
    bo_addrs = base;
    return 0;
}
